@@ -1,23 +1,31 @@
+"""
+Module for jenkinsapi Jenkins object
+"""
+
+
 import json
 import urllib
 import logging
+import urlparse
 
 from jenkinsapi import config
+from jenkinsapi.executors import Executors
 from jenkinsapi.job import Job
+from jenkinsapi.jobs import Jobs
 from jenkinsapi.node import Node
 from jenkinsapi.view import View
 from jenkinsapi.nodes import Nodes
+from jenkinsapi.plugins import Plugins
 from jenkinsapi.views import Views
 from jenkinsapi.queue import Queue
 from jenkinsapi.fingerprint import Fingerprint
 from jenkinsapi.jenkinsbase import JenkinsBase
 from jenkinsapi.utils.requester import Requester
-from jenkinsapi.exceptions import UnknownJob, JenkinsAPIException
-
-
+from jenkinsapi.custom_exceptions import UnknownJob
 
 log = logging.getLogger(__name__)
 
+
 class Jenkins(JenkinsBase):
     """
     Represents a jenkins environment.
@@ -31,11 +39,12 @@ class Jenkins(JenkinsBase):
         """
         self.username = username
         self.password = password
-        self.requester = requester or Requester(username, password)
+        self.requester = requester or Requester(username, password, baseurl=baseurl)
         JenkinsBase.__init__(self, baseurl)
 
     def _clone(self):
-        return Jenkins(self.baseurl, username=self.username, password=self.password, requester=self.requester)
+        return Jenkins(self.baseurl, username=self.username,
+                       password=self.password, requester=self.requester)
 
     def base_server_url(self):
         if config.JENKINS_API in self.baseurl:
@@ -43,17 +52,17 @@ class Jenkins(JenkinsBase):
         else:
             return self.baseurl
 
-    def validate_fingerprint(self, id):
-        obj_fingerprint = Fingerprint(self.baseurl, id, jenkins_obj=self)
+    def validate_fingerprint(self, id_):
+        obj_fingerprint = Fingerprint(self.baseurl, id_, jenkins_obj=self)
         obj_fingerprint.validate()
-        log.info("Jenkins says %s is valid" % id)
+        log.info(msg="Jenkins says %s is valid" % id_)
 
     # def reload(self):
     #     '''Try and reload the configuration from disk'''
     #     self.requester.get_url("%(baseurl)s/reload" % self.__dict__)
 
-    def get_artifact_data(self, id):
-        obj_fingerprint = Fingerprint(self.baseurl, id, jenkins_obj=self)
+    def get_artifact_data(self, id_):
+        obj_fingerprint = Fingerprint(self.baseurl, id_, jenkins_obj=self)
         obj_fingerprint.validate()
         return obj_fingerprint.get_info()
 
@@ -75,12 +84,17 @@ class Jenkins(JenkinsBase):
         # This only ever needs to work on the base object
         return '%s/computer' % self.baseurl
 
+    @property
+    def jobs(self):
+        return Jobs(self)
+
     def get_jobs(self):
         """
         Fetch all the build-names on this Jenkins server.
         """
         for info in self._data["jobs"]:
-            yield info["name"], Job(info["url"], info["name"], jenkins_obj=self)
+            yield info["name"], \
+                Job(info["url"], info["name"], jenkins_obj=self)
 
     def get_jobs_info(self):
         """
@@ -96,7 +110,7 @@ class Jenkins(JenkinsBase):
         :param jobname: name of the job, str
         :return: Job obj
         """
-        return self[jobname]
+        return self.jobs[jobname]
 
     def has_job(self, jobname):
         """
@@ -104,54 +118,28 @@ class Jenkins(JenkinsBase):
         :param jobname: string
         :return: boolean
         """
-        return jobname in self
+        return jobname in self.jobs
 
-    def create_job(self, jobname, config):
+    def create_job(self, jobname, config_):
         """
         Create a job
         :param jobname: name of new job, str
         :param config: configuration of new job, xml
         :return: new Job obj
         """
-        if self.has_job(jobname):
-            return self[jobname]
-
-        params = {'name': jobname}
-        if isinstance(config, unicode):
-            config = str(config)
-        self.requester.post_xml_and_confirm_status(self.get_create_url(), data=config, params=params)
-        self.poll()
-        if not self.has_job(jobname):
-            raise JenkinsAPIException('Cannot create job %s' % jobname)
-        return self[jobname]
+        return self.jobs.create(jobname, config_)
 
     def copy_job(self, jobname, newjobname):
-        """
-        Copy a job
-        :param jobname: name of a exist job, str
-        :param newjobname: name of new job, str
-        :return: new Job obj
-        """
-        params = { 'name': newjobname,
-                   'mode': 'copy',
-                   'from': jobname}
+        return self.jobs.copy(jobname, newjobname)
 
-        self.requester.post_and_confirm_status(
-            self.get_create_url(),
-            params=params,
-            data='')
-        self.poll()
-        return self[newjobname]
-
-    def build_job(self, jobname, params={}):
+    def build_job(self, jobname, params=None):
         """
         Invoke a build by job name
         :param jobname: name of exist job, str
         :param params: the job params, dict
         :return: none
         """
-        self[jobname].invoke(build_params=params)
-        return
+        self[jobname].invoke(build_params=params or {})
 
     def delete_job(self, jobname):
         """
@@ -159,13 +147,7 @@ class Jenkins(JenkinsBase):
         :param jobname: name of a exist job, str
         :return: new jenkins_obj
         """
-        delete_job_url = self[jobname].get_delete_url()
-        self.requester.post_and_confirm_status(
-            delete_job_url,
-            data='some random bytes...'
-        )
-        self.poll()
-        return self
+        del self.jobs[jobname]
 
     def rename_job(self, jobname, newjobname):
         """
@@ -174,12 +156,7 @@ class Jenkins(JenkinsBase):
         :param newjobname: name of new job, str
         :return: new Job obj
         """
-        params = {'newName': newjobname}
-        rename_job_url = self[jobname].get_rename_url()
-        self.requester.post_and_confirm_status(
-            rename_job_url, params=params, data='')
-        self.poll()
-        return self[newjobname]
+        return self.jobs.rename(jobname, newjobname)
 
     def iterkeys(self):
         for info in self._data["jobs"]:
@@ -187,7 +164,8 @@ class Jenkins(JenkinsBase):
 
     def iteritems(self):
         """
-        :param return: An iterator of pairs. Each pair will be (job name, Job object)
+        :param return: An iterator of pairs.
+            Each pair will be (job name, Job object)
         """
         return self.get_jobs()
 
@@ -198,7 +176,7 @@ class Jenkins(JenkinsBase):
         return list(self.get_jobs())
 
     def keys(self):
-        return [ a for a in self.iterkeys() ]
+        return [a for a in self.iterkeys()]
 
     # This is a function alias we retain for historical compatibility
     get_jobs_list = keys
@@ -206,13 +184,20 @@ class Jenkins(JenkinsBase):
     def __str__(self):
         return "Jenkins server at %s" % self.baseurl
 
+    @property
     def views(self):
         return Views(self)
 
     def get_view_by_url(self, str_view_url):
         #for nested view
         str_view_name = str_view_url.split('/view/')[-1].replace('/', '')
-        return View(str_view_url , str_view_name, jenkins_obj=self)
+        return View(str_view_url, str_view_name, jenkins_obj=self)
+
+    def delete_view_by_url(self, str_url):
+        url = "%s/doDelete" % str_url
+        self.requester.post_and_confirm_status(url, data='')
+        self.poll()
+        return self
 
     def __getitem__(self, jobname):
         """
@@ -225,13 +210,16 @@ class Jenkins(JenkinsBase):
                 return Job(info["url"], info["name"], jenkins_obj=self)
         raise UnknownJob(jobname)
 
+    def __len__(self):
+        return len(self._data["jobs"])
+
     def __contains__(self, jobname):
         """
         Does a job by the name specified exist
         :param jobname: string
         :return: boolean
         """
-        return jobname in self.get_jobs_list()
+        return jobname in self.jobs
 
     def get_node(self, nodename):
         """Get a node object for a specific node"""
@@ -239,11 +227,11 @@ class Jenkins(JenkinsBase):
 
     def get_node_url(self, nodename=""):
         """Return the url for nodes"""
-        url = "%(baseurl)s/computer/%(nodename)s" % {'baseurl': self.baseurl, 'nodename': urllib.quote(nodename)}
+        url = urlparse.urljoin(self.base_server_url(), 'computer/%s' % urllib.quote(nodename))
         return url
 
     def get_queue_url(self):
-        url = "%(baseurl)s/queue/" % {'baseurl': self.baseurl}
+        url = "%s/%s" % (self.base_server_url(), 'queue')
         return url
 
     def get_queue(self):
@@ -289,29 +277,53 @@ class Jenkins(JenkinsBase):
         :param exclusive: tied to specific job, boolean
         :return: node obj
         """
-        NODE_TYPE   = 'hudson.slaves.DumbSlave$DescriptorImpl'
+        NODE_TYPE = 'hudson.slaves.DumbSlave$DescriptorImpl'
         MODE = 'NORMAL'
         if self.has_node(name):
             return Node(nodename=name, baseurl=self.get_node_url(nodename=name), jenkins_obj=self)
         if exclusive:
             MODE = 'EXCLUSIVE'
         params = {
-            'name' : name,
-            'type' : NODE_TYPE,
-            'json' : json.dumps ({
-                'name'            : name,
-                'nodeDescription' : node_description,
-                'numExecutors'    : num_executors,
-                'remoteFS'        : remote_fs,
-                'labelString'     : labels,
-                'mode'            : MODE,
-                'type'            : NODE_TYPE,
-                'retentionStrategy' : { 'stapler-class'  : 'hudson.slaves.RetentionStrategy$Always' },
-                'nodeProperties'    : { 'stapler-class-bag' : 'true' },
-                'launcher'          : { 'stapler-class' : 'hudson.slaves.JNLPLauncher' }
+            'name': name,
+            'type': NODE_TYPE,
+            'json': json.dumps({
+                'name': name,
+                'nodeDescription': node_description,
+                'numExecutors': num_executors,
+                'remoteFS': remote_fs,
+                'labelString': labels,
+                'mode': MODE,
+                'type': NODE_TYPE,
+                'retentionStrategy': {'stapler-class': 'hudson.slaves.RetentionStrategy$Always'},
+                'nodeProperties': {'stapler-class-bag': 'true'},
+                'launcher': {'stapler-class': 'hudson.slaves.JNLPLauncher'}
             })
         }
         url = self.get_node_url() + "doCreateItem?%s" % urllib.urlencode(params)
         self.requester.get_and_confirm_status(url)
 
         return Node(nodename=name, baseurl=self.get_node_url(nodename=name), jenkins_obj=self)
+
+    def get_plugins_url(self):
+        # This only ever needs to work on the base object
+        return '%s/pluginManager/api/python?depth=1' % self.baseurl
+
+    def get_plugins(self):
+        url = self.get_plugins_url()
+        return Plugins(url, self)
+
+    def has_plugin(self, plugin_name):
+        return plugin_name in self.get_plugins()
+
+    def get_executors(self, nodename):
+        url = '%s/computer/%s' % (self.baseurl, nodename)
+        return Executors(url, nodename, self)
+
+    @property
+    def version(self):
+        """
+        Return version number of Jenkins
+        """
+        response = self.requester.get_and_confirm_status(self.baseurl)
+        version_key = 'X-Jenkins'
+        return response.headers.get(version_key, '0.0')
