@@ -1,8 +1,14 @@
+"""
+Module for JenkinsBase class
+"""
+
+import ast
 import logging
 from jenkinsapi import config
-from jenkinsapi.exceptions import JenkinsAPIException
+from jenkinsapi.custom_exceptions import JenkinsAPIException
 log = logging.getLogger(__name__)
 
+
 class JenkinsBase(object):
     """
     This appears to be the base object that all other jenkins objects are inherited from
@@ -12,7 +18,7 @@ class JenkinsBase(object):
     def __repr__(self):
         return """<%s.%s %s>""" % (self.__class__.__module__,
                                    self.__class__.__name__,
-                                   str( self ))
+                                   str(self))
 
     def __str__(self):
         raise NotImplementedError
@@ -52,11 +58,11 @@ class JenkinsBase(object):
         url = self.python_api_url(self.baseurl)
         return self.get_data(url)
 
-    def get_data(self, url):
+    def get_data(self, url, params=None):
         requester = self.get_jenkins_obj().requester
-        response = requester.get_url(url)
+        response = requester.get_url(url, params)
         try:
-            return eval(response.text)
+            return ast.literal_eval(response.text)
         except Exception:
             log.exception('Inappropriate content found at %s', url)
             raise JenkinsAPIException('Cannot parse %s' % response.content)
@@ -67,7 +73,7 @@ class JenkinsBase(object):
             return url
         else:
             if url.endswith(r"/"):
-                fmt="%s%s"
+                fmt = "%s%s"
             else:
                 fmt = "%s/%s"
             return fmt % (url, config.JENKINS_API)
