@@ -1,3 +1,7 @@
+"""
+Module for jenkinsapi Job
+"""
+
 import json
 import logging
 import urlparse
@@ -9,7 +13,21 @@ from jenkinsapi.invocation import Invocation
 from jenkinsapi.jenkinsbase import JenkinsBase
 from jenkinsapi.queue import QueueItem
 from jenkinsapi.mutable_jenkins_thing import MutableJenkinsThing
-from jenkinsapi.exceptions import NoBuildData, NotFound, NotInQueue, WillNotBuild, UnknownQueueItem
+from jenkinsapi.custom_exceptions import (
+    NoBuildData,
+    NotConfiguredSCM,
+    NotFound,
+    NotInQueue,
+    NotSupportSCM,
+    WillNotBuild,
+    UnknownQueueItem,
+)
+
+SVN_URL = './scm/locations/hudson.scm.SubversionSCM_-ModuleLocation/remote'
+GIT_URL = './scm/userRemoteConfigs/hudson.plugins.git.UserRemoteConfig/url'
+HG_URL = './scm/source'
+GIT_BRANCH = './scm/branches/hudson.plugins.git.BranchSpec/name'
+HG_BRANCH = './scm/branch'
 
 log = logging.getLogger(__name__)
 
@@ -33,15 +51,15 @@ class Job(JenkinsBase, MutableJenkinsThing):
             'hudson.scm.NullSCM': 'NullSCM'
         }
         self._scmurlmap = {
-            'svn': lambda element_tree: [element for element in element_tree.findall('./scm/locations/hudson.scm.SubversionSCM_-ModuleLocation/remote')],
-            'git': lambda element_tree: [element for element in element_tree.findall('./scm/userRemoteConfigs/hudson.plugins.git.UserRemoteConfig/url')],
-            'hg': lambda element_tree: [element_tree.find('./scm/source')],
+            'svn': lambda element_tree: list(element_tree.findall(SVN_URL)),
+            'git': lambda element_tree: list(element_tree.findall(GIT_URL)),
+            'hg': lambda element_tree: list(element_tree.find(HG_URL)),
             None: lambda element_tree: []
         }
         self._scmbranchmap = {
             'svn': lambda element_tree: [],
-            'git': lambda element_tree: [element for element in element_tree.findall('./scm/branches/hudson.plugins.git.BranchSpec/name')],
-            'hg': lambda element_tree: [element_tree.find('./scm/branch')],
+            'git': lambda element_tree: list(element_tree.findall(GIT_BRANCH)),
+            'hg': lambda element_tree: list(element_tree.find(HG_BRANCH)),
             None: lambda element_tree: []
         }
         JenkinsBase.__init__(self, url)
@@ -55,6 +73,33 @@ class Job(JenkinsBase, MutableJenkinsThing):
     def get_jenkins_obj(self):
         return self.jenkins
 
+    def _poll(self):
+        data = JenkinsBase._poll(self)
+        # jenkins loads only the first 100 builds, load more if needed
+        data = self._add_missing_builds(data)
+        return data
+
+    # pylint: disable=E1123
+    # Unexpected keyword arg 'params'
+    def _add_missing_builds(self, data):
+        '''Query Jenkins to get all builds of the job in the data object.
+
+        Jenkins API loads the first 100 builds and thus may not contain all builds
+        information. This method checks if all builds are loaded in the data object
+        and updates it with the missing builds if needed.'''
+        if not (data.get("builds") and data.get("firstBuild")):
+            return data
+        # do not call _buildid_for_type here: it would poll and do an infinite loop
+        oldest_loaded_build_number = data["builds"][-1]["number"]
+        first_build_number = data["firstBuild"]["number"]
+        all_builds_loaded = (oldest_loaded_build_number == first_build_number)
+        if all_builds_loaded:
+            return data
+        api_url = self.python_api_url(self.baseurl)
+        response = self.get_data(api_url, params={'tree': 'allBuilds[number,url]'})
+        data['builds'] = response['allBuilds']
+        return data
+
     def _get_config_element_tree(self):
         """
         The ElementTree objects creation is unnecessary, it can be a singleton per job
@@ -66,7 +111,9 @@ class Job(JenkinsBase, MutableJenkinsThing):
             self._element_tree = ET.fromstring(self._config)
         return self._element_tree
 
-    def get_build_triggerurl(self):
+    def get_build_triggerurl(self, build_params=None, files=None):
+        if build_params or files:
+            return "%s/buildWithParameters" % self.baseurl
         return "%s/build" % self.baseurl
 
     @staticmethod
@@ -86,7 +133,8 @@ class Job(JenkinsBase, MutableJenkinsThing):
         to_json_structure = Job._mk_json_from_build_parameters(build_params)
         return json.dumps(to_json_structure)
 
-    def invoke(self, securitytoken=None, block=False, skip_if_running=False, invoke_pre_check_delay=3, invoke_block_delay=15, build_params=None, cause=None):
+    def invoke(self, securitytoken=None, block=False, skip_if_running=False, invoke_pre_check_delay=3,
+               invoke_block_delay=15, build_params=None, cause=None, files=None):
         assert isinstance(invoke_pre_check_delay, (int, float))
         assert isinstance(invoke_block_delay, (int, float))
         assert isinstance(block, bool)
@@ -101,21 +149,25 @@ class Job(JenkinsBase, MutableJenkinsThing):
         params = {}  # Via Get string
 
         with invocation:
-            if self.is_queued():
-                raise WillNotBuild('%s is already queued' % repr(self))
+            if len(self.get_params_list()) == 0:
+                if self.is_queued():
+                    raise WillNotBuild('%s is already queued' % repr(self))
 
-            elif self.is_running():
-                if skip_if_running:
-                    log.warn(
-                        "Will not request new build because %s is already running", self.name)
-                else:
-                    log.warn(
-                        "Will re-schedule %s even though it is already running", self.name)
+                elif self.is_running():
+                    if skip_if_running:
+                        log.warn(
+                            "Will not request new build because %s is already running", self.name)
+                    else:
+                        log.warn(
+                            "Will re-schedule %s even though it is already running", self.name)
+            elif self.has_queued_build(build_params):
+                msg = 'A build with these parameters is already queued.'
+                raise WillNotBuild(msg)
 
             log.info("Attempting to start %s on %s", self.name, repr(
                 self.get_jenkins_obj()))
 
-            url = self.get_build_triggerurl()
+            url = self.get_build_triggerurl(build_params, files)
 
             if cause:
                 build_params['cause'] = cause
@@ -123,13 +175,16 @@ class Job(JenkinsBase, MutableJenkinsThing):
             if securitytoken:
                 params['token'] = securitytoken
 
+            data = build_params
+
             response = self.jenkins.requester.post_and_confirm_status(
                 url,
-                data={'json': self.mk_json_from_build_parameters(
-                    build_params)},  # See above - build params have to be JSON encoded & posted.
+                data=data,
                 params=params,
+                files=files,
                 valid=[200, 201]
             )
+            response = response
             if invoke_pre_check_delay > 0:
                 log.info(
                     "Waiting for %is to allow Jenkins to catch up", invoke_pre_check_delay)
@@ -149,9 +204,10 @@ class Job(JenkinsBase, MutableJenkinsThing):
         return invocation
 
     def _buildid_for_type(self, buildtype):
-        self.poll()
         """Gets a buildid for a given type of build"""
+        self.poll()
         KNOWNBUILDTYPES = [
+            "lastStableBuild",
             "lastSuccessfulBuild",
             "lastBuild",
             "lastCompletedBuild",
@@ -159,7 +215,7 @@ class Job(JenkinsBase, MutableJenkinsThing):
             "lastFailedBuild"]
         assert buildtype in KNOWNBUILDTYPES, 'Unknown build info type: %s' % buildtype
 
-        if not self._data[buildtype]:
+        if not self._data.get(buildtype):
             raise NoBuildData(buildtype)
         return self._data[buildtype]["number"]
 
@@ -169,19 +225,25 @@ class Job(JenkinsBase, MutableJenkinsThing):
         """
         return self._buildid_for_type("firstBuild")
 
+    def get_last_stable_buildnumber(self):
+        """
+        Get the numerical ID of the last stable build.
+        """
+        return self._buildid_for_type("lastStableBuild")
+
     def get_last_good_buildnumber(self):
         """
         Get the numerical ID of the last good build.
         """
         return self._buildid_for_type("lastSuccessfulBuild")
 
-    def get_last_failed_buildnumber( self ):
+    def get_last_failed_buildnumber(self):
         """
         Get the numerical ID of the last good build.
         """
         return self._buildid_for_type(buildtype="lastFailedBuild")
 
-    def get_last_buildnumber( self ):
+    def get_last_buildnumber(self):
         """
         Get the numerical ID of the last build.
         """
@@ -194,10 +256,16 @@ class Job(JenkinsBase, MutableJenkinsThing):
         return self._buildid_for_type("lastCompletedBuild")
 
     def get_build_dict(self):
-        if not self._data.has_key("builds"):
+        if "builds" not in self._data:
             raise NoBuildData(repr(self))
-        return dict((build["number"], build["url"])
-                        for build in self._data["builds"])
+        builds = self._data["builds"]
+        last_build = self._data['lastBuild']
+        if builds and last_build and builds[0]['number'] != last_build['number']:
+            builds = [last_build] + builds
+        # FIXME SO how is this supposed to work if build is false-y?
+        # I don't think that builds *can* be false here, so I don't
+        # understand the test above.
+        return dict((build["number"], build["url"]) for build in builds)
 
     def get_revision_dict(self):
         """
@@ -223,6 +291,13 @@ class Job(JenkinsBase, MutableJenkinsThing):
         """
         return self._data.get('nextBuildNumber', 0)
 
+    def get_last_stable_build(self):
+        """
+        Get the last stable build
+        """
+        bn = self.get_last_stable_buildnumber()
+        return self.get_build(bn)
+
     def get_last_good_build(self):
         """
         Get the last good build
@@ -281,6 +356,9 @@ class Job(JenkinsBase, MutableJenkinsThing):
     def __getitem__(self, buildnumber):
         return self.get_build(buildnumber)
 
+    def __len__(self):
+        return len(self.get_build_dict())
+
     def is_queued_or_running(self):
         return self.is_queued() or self.is_running()
 
@@ -321,10 +399,10 @@ class Job(JenkinsBase, MutableJenkinsThing):
         scm_class = element_tree.find('scm').get('class')
         scm = self._scm_map.get(scm_class)
         if not scm:
-            raise exceptions.NotSupportSCM(
+            raise NotSupportSCM(
                 "SCM class \"%s\" not supported by API, job \"%s\"" % (scm_class, self.name))
         if scm == 'NullSCM':
-            raise exceptions.NotConfiguredSCM(
+            raise NotConfiguredSCM(
                 "SCM does not configured, job \"%s\"" % self.name)
         return scm
 
@@ -513,3 +591,12 @@ class Job(JenkinsBase, MutableJenkinsThing):
         for param in self.get_params():
             params.append(param['name'])
         return params
+
+    def has_queued_build(self, build_params):
+        """Returns True if a build with build_params is currently queued."""
+        queue = self.jenkins.get_queue()
+        queued_builds = queue.get_queue_items_for_job(self.name)
+        for build in queued_builds:
+            if build.get_parameters() == build_params:
+                return True
+        return False
