@@ -1,11 +1,14 @@
+"""
+Queue module for jenkinsapi
+"""
+
 from jenkinsapi.jenkinsbase import JenkinsBase
-from jenkinsapi.exceptions import UnknownQueueItem
+from jenkinsapi.custom_exceptions import UnknownQueueItem
 import logging
-import urlparse
-import urllib2
 
 log = logging.getLogger(__name__)
 
+
 class Queue(JenkinsBase):
     """
     Class that represents the Jenkins queue
@@ -30,37 +33,43 @@ class Queue(JenkinsBase):
         for item in self._data['items']:
             yield item['id'], QueueItem(self.jenkins, **item)
 
+    def iterkeys(self):
+        for item in self._data['items']:
+            yield item['id']
+
+    def iterivalues(self):
+        for item in self._data['items']:
+            yield QueueItem(self.jenkins, **item)
+
     def keys(self):
-        return [i[0] for i in self.iteritems()]
+        return list(self.iterkeys())
+
+    def values(self):
+        return list(self.itervalues())
 
     def __len__(self):
         return len(self._data['items'])
 
     def __getitem__(self, item_id):
-        for id, item in self.iteritems():
-            if id == item_id:
-                return item
-        raise UnknownQueueItem(item_id)
+        self_as_dict = dict(self.iteritems())
+        if item_id in self_as_dict:
+            return self_as_dict[item_id]
+        else:
+            raise UnknownQueueItem(item_id)
 
     def get_queue_items_for_job(self, job_name):
         if not job_name:
-            return [QueueItem(**item) for item in self._data['items']]
+            return [QueueItem(self.jenkins, **item) for item in self._data['items']]
         else:
-            return [QueueItem(**item) for item in self._data['items']
-                   if item['task']['name'] == job_name]
+            return [QueueItem(self.jenkins, **item) for item in self._data['items']
+                    if item['task']['name'] == job_name]
 
     def delete_item(self, queue_item):
         self.delete_item_by_id(queue_item.id)
 
     def delete_item_by_id(self, item_id):
-        deleteurl = urlparse.urljoin(self.baseurl,
-                                     'cancelItem?id=%s' % item_id)
-        try:
-            self.post_data(deleteurl, '')
-        except urllib2.HTTPError:
-            # The request doesn't have a response, so it returns 404,
-            # it's the expected behaviour
-            pass
+        deleteurl = '%s/cancelItem?id=%s' % (self.baseurl, item_id)
+        self.get_jenkins_obj().requester.post_url(deleteurl)
 
 
 class QueueItem(object):
@@ -79,11 +88,17 @@ class QueueItem(object):
         """
         return self.jenkins[self.task['name']]
 
+    def get_parameters(self):
+        """returns parameters of queue item"""
+        actions = getattr(self, 'actions', [])
+        for action in actions:
+            if type(action) is dict and 'parameters' in action:
+                parameters = action['parameters']
+                return dict([(x['name'], x['value']) for x in parameters])
+        return []
+
     def __repr__(self):
-        return "<%s.%s %s>" % (
-                    self.__class__.__module__,
-                    self.__class__.__name__,
-                    str(self))
+        return "<%s.%s %s>" % (self.__class__.__module__, self.__class__.__name__, str(self))
 
     def __str__(self):
         return "%s #%i" % (self.task['name'], self.id)
