@@ -1,23 +1,24 @@
+"""
+Module for jenkinsapi Views
+"""
+
 import logging
 import json
 from jenkinsapi.view import View
-from jenkinsapi.exceptions import UnknownView
 
 log = logging.getLogger(__name__)
 
-class Views(object):
 
-    # TODO @lechat 20130702: Add check that plugin for view actually exists in Jenkins
+class Views(object):
+    """
+    An abstraction on a Jenkins object's views
+    """
     LIST_VIEW = 'hudson.model.ListView'
     NESTED_VIEW = 'hudson.plugins.nested_view.NestedView'
     MY_VIEW = 'hudson.model.MyView'
     DASHBOARD_VIEW = 'hudson.plugins.view.dashboard.Dashboard'
     PIPELINE_VIEW = 'au.com.centrumsystems.hudson.plugin.buildpipeline.BuildPipelineView'
 
-    """
-    An abstraction on a Jenkins object's views
-    """
-
     def __init__(self, jenkins):
         self.jenkins = jenkins
 
@@ -32,25 +33,27 @@ class Views(object):
             self[view_name].delete()
             self.jenkins.poll()
 
-    def __setitem__(self, name, value):
-        raise NotImplementedError()
+    def __setitem__(self, view_name, job_names_list):
+        new_view = self.create(view_name)
+        if isinstance(job_names_list, str):
+            job_names_list = [job_names_list]
+        for job_name in job_names_list:
+            if not new_view.add_job(job_name):
+                # Something wrong - delete view
+                del self[new_view]
+                raise TypeError('Job %s does not exist in Jenkins' % job_name)
 
     def __getitem__(self, view_name):
-        for row in self.jenkins._data['views']:
+        for row in self.jenkins._data.get('views', []):
             if row['name'] == view_name:
-                return View(
-                    row['url'],
-                    row['name'],
-                    self.jenkins)
-        else:
-            return None
+                return View(row['url'], row['name'], self.jenkins)
 
     def iteritems(self):
         """
         Get the names & objects for all views
         """
         self.jenkins.poll()
-        for row in self.jenkins._data['views']:
+        for row in self.jenkins._data.get('views', []):
             name = row['name']
             url = row['url']
 
@@ -66,7 +69,7 @@ class Views(object):
         """
         Get the names of all available views
         """
-        for row in self.jenkins._data['views']:
+        for row in self.jenkins._data.get('views', []):
             yield row['name']
 
     def keys(self):
@@ -75,12 +78,6 @@ class Views(object):
         """
         return list(self.iterkeys())
 
-    def delete_view_by_url(self, str_url):
-        url = "%s/doDelete" % str_url
-        response = self.requester.get_url(url, data='')
-        self.jenkins.poll()
-        return self
-
     def create(self, view_name, view_type=LIST_VIEW):
         """
         Create a view
@@ -88,11 +85,11 @@ class Views(object):
         :param person: Person name (to create personal view), str
         :return: new View obj or None if view was not created
         """
-        log.info('Creating "%s" view "%s"' % (view_type, view_name))
+        log.info(msg='Creating "%s" view "%s"' % (view_type, view_name))
         #url = urlparse.urljoin(self.baseurl, "user/%s/my-views/" % person) if person else self.baseurl
 
         if view_name in self:
-            log.warn('View "%s" already exists' % view_name)
+            log.warn(msg='View "%s" already exists' % view_name)
             return self[view_name]
 
         url = '%s/createView' % self.jenkins.baseurl
